<?php

namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;

class Consumer extends \Zeedhi\Framework\Controller\Simple {
    
    
    const ERROR_MESSAGE = 'E';
    const status = array(
        '1' => 'Em Aberto',
        '2' => 'Conta Solicitada',
        '3' => 'Pago',
        '4' => 'Cancelado',
        '5' => 'Transferido/Despachado',
        '6' => 'Não Pago',
        'P' => 'Impresso',
        'X' => 'Impresso',
        'A' => 'Agendado',
        'O' => 'Concluido'
    );
    
    protected $util;
    protected $consumerService;
    protected $orderService;
    protected $consumerAPI;
    
    public function __construct(\Util\Util $util, \Service\Consumer $consumerService, \Service\Order $orderService,
                                \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager,
                                \Odhen\API\Service\Consumidor $consumerAPI) {
        $this->util            = $util;
        $this->consumerService = $consumerService;
        $this->orderService    = $orderService;
        date_default_timezone_set($instanceManager->getParameter('TIMEZONE'));
        $this->consumerAPI     = $consumerAPI;
    }
    
    /* 
        Retorna os consumidores alterados ou inseridos em um período especificado. 
    */
    
    public function getConsumersByPeriod(Request $request, Response $response) {
        
        $inputJSON = file_get_contents('php://input');
        $input     = json_decode($inputJSON, true);
        
        $this->util->beginTransaction();
        
        try {
            
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // Dados a serem retornados
            $params = $input;
            $from   = $params['from'];
            $to     = $params['to'];
            
            // verifica o intervalo de tempo
            if (!($this->util->isValidDateInterval($from, $to))) {
                throw new \Exception('Intervalo de datas inválido.');
            }
            
            $return = $this->consumerService->getConsumersByPeriod($from, $to);
            
            if (!$return['error']) {
                $consumers = $return['data'];
                if (!empty($consumers)) {
                    
                    // formata os dados retornados para cada consumidor
                    foreach ($consumers as $consumer) {
                        $mainAddress = array(
                            'identifier'    => 'P',
                            'description'   => 'Principal',
                            'zipCode'       => $consumer['NRCEPCONS'],
                            'streetAddress' => $consumer['DSENDECONS'],
                            'number'        => $consumer['NRENDECONS'],
                            'city'          => $consumer['CDMUNICIBGE'],
                            'state'         => $consumer['SGESTADO'],
                            'country'       => $consumer['NMPAIS'],
                            'addressLine2'  => $consumer['DSCOMPLENDECONS'],
                            'neighborhood'  => $consumer['NMBAIRRO'],
                            'phone'         => $consumer['NRTELECONS'],
                            'cellphone'     => $consumer['NRCELULARCONS'],
                            'isPrincipal'   => true
                        );
                        
                        $consumerData = array(
                            'clientCode'        => $consumer['CDCLIENTE'],    
                            'consumerCode'      => $consumer['CDCONSUMIDOR'],
                            'consumerType'      => $consumer['CDTIPOCONS'],
                            'costCenter'        => $consumer['CDCCUSCLIE'],
                            'cpf'               => $consumer['NRCPFRESPCON'],
                            'consumerName'      => $consumer['NMCONSUMIDOR'],
                            'updateDate'        => $consumer['DTULTATUCSUM'], 
                            'email'             => $consumer['DSEMAILCONS'],
                            'consumerBirthDate' => $consumer['DTNASCCONS'],
                            'addresses'         => array($mainAddress),
                            'isActive'          => $consumer['IDATIVO'] == 1 || $consumer['IDATIVO'] == "1" ,
                            'gender'            => $consumer['IDSEXOCONS']
                        );

                        // busca por endereços adicionais do consumidor
                        $return = $this->consumerService->getConsumerAditionalAddresses($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);
                        
                        if (!$return['error']) {
                            $aditionalAddresses = $return['data'];
                            
                            // formata dados retornados para cada endereço adicional
                            for ($i = 0; $i < sizeof($aditionalAddresses); $i++) {
                                $address = $aditionalAddresses[$i];
                                
                                $identifier = 'O'.$i;
                                $addressData = array(
                                    'identifier'    => $identifier,
                                    'description'   => $address['NOMEENDE'],
                                    'zipCode'       => $address['NRCEPCONS'],
                                    'streetAddress' => $address['DSENDECONS'],
                                    'number'        => $address['NRENDECONS'],
                                    'city'          => $address['CDMUNICIBGE'],
                                    'state'         => $address['SGESTADO'],
                                    'country'       => $address['NMPAIS'],
                                    'addressLine2'  => $address['DSCOMPLENDECONS'],
                                    'neighborhood'  => $address['NMBAIRRO'],
                                    'phone'         => '',
                                    'cellphone'     => '',
                                    'isPrincipal'   => false
                                );
        
                                array_push($consumerData['addresses'], $addressData);
                            }
                        }
                
                        array_push($data, $consumerData);
                    }
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    $message = 'Nenhum consumidor foi encontrado no período.';
                    $response->addMessage(new Message($message));
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao buscar consumidores. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function insertExtractMovement(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        
        try {

            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            } 
            $data   = array(); // dados a serem retornados
            $params = $input;
            
            $clientCode     = $params['clientCode'];
            $consumerCode   = $params['consumerCode'];
            $description    = $params['description'];
            $value          = $this->util->float($params['value']);
            $movementType   = $params['movementType'];
            $family         = $params['family'];
            $subsidiaryCode = $params['subsidiaryCode'];

            if(($params['movementType'] !== "C" && $params['movementType'] !== "D")){
                throw new \Exception("Parâmetro movementType deve ser preenchido com 'C' para Crédito ou 'D' para Débito.");
            }
            // verifica se existe cliente cadastrado com essas informações
            if (!$this->util->consumerExists($clientCode, $consumerCode)) {
                throw new \Exception('Consumidor inexistente.');
            } 
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            $this->saveExtratocons($clientCode, $consumerCode, $family, $description, $value, $movementType, date('d/m/Y H:i:s'));
            $message = "Operação realizada com sucesso, valor: $value";
            $response->addMessage(new Message($message));

        }catch(\Exception $e){
            $message = 'Erro ao inserir movimentacao. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    public function getConsumerById(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true); 
        $this->util->beginTransaction();
        try {
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados
            
            $params       = $input;
            $consumerCode = $params['consumerCode'];
            $return = $this->consumerService->getConsumerById($consumerCode);
            if (!$return['error']) {
                $consumer = $return['data'];
                if (!empty($consumer)) {
                    // formata os dados retornados para cada consumidor
                    $mainAddress = array(
                        'identifier'    => 'P',
                        'description'   => 'Principal',
                        'zipCode'       => $consumer['NRCEPCONS'],
                        'streetAddress' => $consumer['DSENDECONS'],
                        'number'        => $consumer['NRENDECONS'],
                        'city'          => $consumer['CDMUNICIBGE'],
                        'state'         => $consumer['SGESTADO'],
                        'country'       => $consumer['NMPAIS'],
                        'addressLine2'  => $consumer['DSCOMPLENDECONS'],
                        'neighborhood'  => $consumer['NMBAIRRO'],
                        'phone'         => $consumer['NRTELECONS'],
                        'cellphone'     => $consumer['NRCELULARCONS'],
                        'isPrincipal'   => true
                    );
                    
                    $consumerData = array(   
                        'consumerCode'      => $consumer['CDCONSUMIDOR'],
                        'cpf'               => $consumer['NRCPFRESPCON'],
                        'consumerName'      => $consumer['NMCONSUMIDOR'],
                        'updateDate'        => $consumer['DTULTATUCSUM'],
                        'email'             => $consumer['DSEMAILCONS'],
                        'consumerBirthDate' => $consumer['DTNASCCONS'],
                        'addresses'         => array($mainAddress),
                        'consumerGroups'    => array(),
                        'isActive'          => $consumer['IDATIVO'] == 1 || $consumer['IDATIVO'] == "1" ,
                        'gender'            => $consumer['IDSEXOCONS']
                    );

                    // Busca por grupos de consumo do consumidor

                    $return = $this->consumerService->getConsumerGroups($consumerCode);
                    if (!$return['error']) {
                        $consumerGroups = $return['data'];
                        if(!empty($consumerGroups)){
                            // formata dados retornados para cada endereço adicional
                            foreach ($consumerGroups as $consumerGroup){

                                $consumerGroupData = array(
                                    'clientCode'      => $consumerGroup['CDCLIENTE'],
                                    'consumerType'    => $consumerGroup['CDTIPOCONS'],
                                    'costCenter'      => $consumerGroup['CDCCUSCLIE'],
                                    'updateDate'      => $consumerGroup['DTULTATUCSUM'],
                                    'typeDescription' => $consumerGroup['NMTIPOCONS'],
                                    'isActive'        => $consumerGroup['IDATIVO'] == 1 || $consumerGroup['IDATIVO'] == "1"
                                );
        
                                array_push($consumerData['consumerGroups'], $consumerGroupData);
                            }
                        }    
                    }
                    array_push($data, $consumerData);
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    $message = 'Nenhum consumidor foi encontrado.';
                    $response->addMessage(new Message($message));
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao buscar consumidor. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function getConsumerByCpf(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true); 
        $this->util->beginTransaction();
        
        try {
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados
            
            $params             = $input;
            $consumerCpf        = $params['cpf'];
            $return             = $this->consumerService->getConsumerByCpf($consumerCpf);
            if (!$return['error']) {
                $consumer = $return['data'];
                if (!empty($consumer)) {
                    // formata os dados retornados para cada consumidor
                    $mainAddress = array(
                        'identifier'    => 'P',
                        'description'   => 'Principal',
                        'zipCode'       => $consumer['NRCEPCONS'],
                        'streetAddress' => $consumer['DSENDECONS'],
                        'number'        => $consumer['NRENDECONS'],
                        'city'          => $consumer['CDMUNICIBGE'],
                        'state'         => $consumer['SGESTADO'],
                        'country'       => $consumer['NMPAIS'],
                        'addressLine2'  => $consumer['DSCOMPLENDECONS'],
                        'neighborhood'  => $consumer['NMBAIRRO'],
                        'phone'         => $consumer['NRTELECONS'],
                        'cellphone'     => $consumer['NRCELULARCONS'],
                        'isPrincipal'   => true
                    );
                    
                    $consumerData = array(   
                        'cpf'               => $consumer['NRCPFRESPCON'],
                        'consumerName'      => $consumer['NMCONSUMIDOR'],
                        'updateDate'        => $consumer['DTULTATUCSUM'],
                        'email'             => $consumer['DSEMAILCONS'],
                        'consumerBirthDate' => $consumer['DTNASCCONS'],
                        'addresses'         => array($mainAddress),
                        'consumerGroups'    => array(),
                        'isActive'          => $consumer['IDATIVO'] == 1 || $consumer['IDATIVO'] == "1" ,
                        'gender'            => $consumer['IDSEXOCONS']
                    );

                    // Busca por grupos de consumo do consumidor

                    $return = $this->consumerService->getConsumerGroupsByCpf($consumerCpf);
                    if (!$return['error']) {
                        $consumerGroups = $return['data'];
                        if(!empty($consumerGroups)){
                            // formata dados retornados para cada endereço adicional
                            foreach ($consumerGroups as $consumerGroup){
                                if(!empty($consumerGroup['CDFAMILISALD']))
                                    $balanceValue = $this->util->getConsumerBalance($consumerGroup['CDCLIENTE'], $consumerGroup['CDCONSUMIDOR'], $consumerGroup['CDFAMILISALD']);
                                $consumerGroupData = array(
                                    'clientCode'      => $consumerGroup['CDCLIENTE'],
                                    'consumerType'    => $consumerGroup['CDTIPOCONS'],
                                    'consumerCode'    => $consumerGroup['CDCONSUMIDOR'],
                                    'costCenter'      => $consumerGroup['CDCCUSCLIE'],
                                    'balanceValue'    => !empty($balanceValue) ? $balanceValue : 0,
                                    'updateDate'      => $consumerGroup['DTULTATUCSUM'],
                                    'typeDescription' => $consumerGroup['NMTIPOCONS'],
                                    'isActive'        => $consumerGroup['IDATIVO'] == 1 || $consumerGroup['IDATIVO'] == "1"
                                );
                                array_push($consumerData['consumerGroups'], $consumerGroupData);
                            }
                        }    
                    }
                    array_push($data, $consumerData);
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    $message = 'Nenhum consumidor foi encontrado.';
                    $response->addMessage(new Message($message));
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao buscar consumidor. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    public function saveConsumer(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true); 
        $this->util->beginTransaction();
        try {
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // Dados a serem retornados
            $consumer = $input;
           
            // Verifica se a data de nascimento foi enviada no formato esperado
            if (!empty($consumer['consumerBirthDate'])  && !$this->util->isValidDate($consumer['consumerBirthDate']))
                throw new \Exception('Formato da data de nascimento inválido.');
            
            // Verifica se o gênero do consumidor é válido
            if (!empty($consumer['gender']) && !($consumer['gender'] == 'M' || $consumer['gender'] == 'F'))
                throw new \Exception('O gênero do consumidor é inválido.');
                
            $consumerData = array(
                'CDCLIENTE'       => $consumer['clientCode'],
                'CDCONSUMIDOR'    => $consumer['consumerCode'],
                'CDTIPOCONS'      => $consumer['consumerType'],
                'CDCCUSCLIE'      => $consumer['costCenter'],
                'NRCPFRESPCON'    => $consumer['cpf'],
                'NMCONSUMIDOR'    => $consumer['consumerName'],
                'DSEMAILCONS'     => $consumer['email'],
                'DTNASCCONS'      => $consumer['consumerBirthDate'],
                'IDATIVO'         => $consumer['isActive'] ? '1' : '2',
                'IDSEXOCONS'      => $consumer['gender'],
                'NRCEPCONS'       => '',
                'DSENDECONS'      => '',
                'NRENDECONS'      => '',
                'CDMUNICIPIO'     => '',
                'SGESTADO'        => '',
                'CDPAIS'          => '',
                'DSCOMPLENDECONS' => '',
                'CDBAIRRO'        => '',
                'NRTELECONS'      => '',
                'NRCELULARCONS'   => ''
            );
            
            
            $addresses = !empty($consumer['addresses']) ? $consumer['addresses'] : array();
            $mainAddress        = array();
            $aditionalAddresses = array();
            
            // Verifica se o consumidor possui endereços relacionados
            if (sizeof($addresses) > 0) {
                // Endereço principal
                $mainAddress = array_filter($addresses, function($address) {
                    return $address['isPrincipal'] == '1'|| $address['isPrincipal'] ;
                });
                
                // Endereço(s) secundário(s)
                $aditionalAddresses = array_filter($addresses, function($address) {
                    return $address['isPrincipal'] == '0' || !$address['isPrincipal'] ;
                });
            }
            
            // Adiciona as informações do endereço principal às informações do consumidor
            if (!empty($mainAddress)) {
                $mainAddress = $mainAddress[0];
                $cityCode    = null;
                if($mainAddress['city'] != null){
                    $cityCodeExt = $mainAddress['city'];
                    $stateCode   = $mainAddress['state'];
                    $cityCode    = $this->util->getCityCode($cityCodeExt,$stateCode);
                }
                $consumerData['NRCEPCONS']       = $mainAddress['zipCode'];
                $consumerData['DSENDECONS']      = $mainAddress['streetAddress'];
                $consumerData['NRENDECONS']      = $mainAddress['number'];
                $consumerData['CDMUNICIPIO']     = $cityCode;
                $consumerData['SGESTADO']        = $mainAddress['state'];
                $consumerData['CDPAIS']          = '0055';
                $consumerData['DSCOMPLENDECONS'] = $mainAddress['addressLine2'];
                $consumerData['DSBAIRRO']        = $mainAddress['neighborhood'];
                $consumerData['NRTELECONS']      = $mainAddress['phone'];
                $consumerData['NRCELULARCONS']   = $mainAddress['cellphone'];
            }
            
            // Verifica se já existe algum consumidor cadastrado com os códigos recebidos
            if ($this->util->consumerExists($consumer['clientCode'], $consumer['consumerCode'])) {
                // Atualiza o consumidor
                $return = $this->consumerService->editConsumer($consumerData);
            }else{
                // Cadastra o consumidor
                $return = $this->consumerService->saveConsumer($consumerData);
            }
            
            if ($return['error']) {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }else
                $consumer['consumerCode'] = !empty($consumer['consumerCode']) ? $consumer['consumerCode'] : $return['data']['consumerCode'];
            
            // Verifica se o consumidor possui endereços adicionais
            if (!empty($aditionalAddresses)) {
                // Cadastra os endereços adicionais do consumidor
                foreach ($aditionalAddresses as $address) {
                    // Verifica se os campos obrigatórios estão preenchidos
                    if (empty($address['identifier'])) {
                        throw new \Exception('Parâmetro identifier é obrigatório.');
                    }
                    if (empty($address['description'])) {
                        throw new \Exception('Parâmetro description é obrigatório.');
                    }
                    if (empty($address['zipCode'])) {
                        throw new \Exception('Parâmetro zipCode é obrigatório.');
                    }
                    if (empty($address['streetAddress'])) {
                        throw new \Exception('Parâmetro streetAddress é obrigatório.');
                    }
                    if (empty($address['number'])) {
                        throw new \Exception('Parâmetro number é obrigatório.');
                    }
                    if (empty($address['country'])) {
                        $address['country'] = '0055';
                    }
                    
                    $addressData = array(
                        'CDCLIENTE'       => $consumer['clientCode'],
                        'CDCONSUMIDOR'    => $consumer['consumerCode'],
                        'CDENDECONS'      => $address['identifier'],
                        'NOMEENDE'        => $address['description'],
                        'NRCEPCONS'       => $address['zipCode'],
                        'DSENDECONS'      => $address['streetAddress'],
                        'NRENDECONS'      => $address['number'],
                        'CDMUNICIPIO'     => $address['city'],
                        'SGESTADO'        => $address['state'],
                        'CDPAIS'          => $address['country'],
                        'DSCOMPLENDECONS' => $address['addressLine2'],
                        'DSBAIRRO'        => $address['neighborhood']
                    );
                    // Cadastra endereços adicionais
                    $return = $this->consumerService->addAdditionalAddress($addressData);
                    if ($return['error']) {
                        $message = $return['message'];
                        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                    }
                }
            }
            array_push($data, $consumer);
            
            $response->addDataSet(new DataSet('', $data));
            $this->util->commit();
        } catch(\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao cadastrar consumidor. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    /* 
        Retorna todas as vendas aprovadas de um consumidor, para um período especificado. 
    */
    public function getSalesByPeriod(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        //$this->util->saveLog($inputJSON,'consumer','getSalesByPeriod');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            $data   = array(); // Dados a serem retornados
            
            $params       = $input;
            $clientCode   = $params['clientCode'];
            $consumerCode = $params['consumerCode'];
            
            $from          = $params['from'];
            $to            = $params['to'];
            $checkDate     = 'T'; 
            $counter       = $params['counter'];
            $checkCounter  = 'T';
            $subsidiaries  = !empty($params['subsidiaries']) ? $params['subsidiaries'] : $this->util->getSubsidiaries(); 
            // Verifica se existe cliente cadastrado com essas informações
            if (!$this->util->consumerExists($clientCode, $consumerCode)) {
                throw new \Exception('Consumidor inexistente.');
            }
            
            if(!empty($from) && !empty($to)){
                // Verifica se as datas foram enviadas no formato requerido
                if (!($this->util->isValidDate($from) && $this->util->isValidDate($to))) {
                    throw new \Exception('Formato inválido de datas');
                }
                
                // Verifica o intervalo de tempo
                if (!($this->util->isValidDateInterval($from, $to))) {
                    throw new \Exception('Intervalo de datas inválido.');
                }
            }else{
                $checkDate = 'F';
                $from = date('d/m/Y');
                $to   = date('d/m/Y');
            } 

            if(empty($counter)){
                $counter = 1;
                $checkCounter = 'F';
            }
            $return = $this->consumerService->hasOpenSale($consumerCode, $clientCode, $subsidiaries);
            if (!$return['error']) {
                $openSales = $return['data'];
                foreach ($openSales as $openSale) {
                    $openSaleId = $this->util->getBillExtId($openSale['NRCOMANDA'],$openSale['NRVENDAREST'], $openSale['CDFILIAL']);
                    if(!empty($openSaleId)){
                        $return = $this->consumerService->getLastSale($openSale['NRCOMANDA'],$openSale['CDFILIAL'], $clientCode, $consumerCode);
                        if (!$return['error']) {
                            $isComandaVen = $return['isComandaVen'];
                            $lastSale = $return['data'];
                            
                            if (!empty($lastSale)) {
                                // Formata os dados da última venda aprovada do cliente
                                $deliveryFee   = $lastSale['IDRETBALLOJA'] == 'S' ? 0 : $lastSale['VRACRCOMANDA'];
                                $addressAsArray = explode(',',$lastSale['DSENDECONSCOMAN']);
                                $streetAddress = $addressAsArray[0] !== "" ? $addressAsArray[0] : null;
                                $number = null;
                                if(count($addressAsArray) > 1){
                                    $number = $addressAsArray[1];
                                }
                                
                                $orderDiscount = $this->util->removeDecimal($lastSale['VRDESCOMANDA']);
                                $lastSaleData = array(
                                    'subsidiaryCode'   => $lastSale['CDFILIAL'],
                                    'clientCode'       => $lastSale['CDCLIENTE'],
                                    'consumerCode'     => $lastSale['CDCONSUMIDOR'],
                                    'saleDate'         => $lastSale['DTVENDA'],
                                    'saleCode'         => $lastSale['NRCOMANDA'],
                                    'saleId'           => $lastSale['NRCOMANDAEXT'],
                                    'orderStatus'      => self::status[$lastSale['IDSTCOMANDA']],
                                    'isOpen'           => !!$lastSale['ISOPEN'] ? 'true' : 'false',
                                    'items'            => array(),
                                    'orderDescription' => array(
                                        'orderType'     => $lastSale['IDORIGEMVENDA'],
                                        'productsValue' => 0,
                                        'deliveryFee'   => $deliveryFee,
                                        'orderDiscount' => $orderDiscount,
                                        'totalValue'    => 0,
                                        'description'   => '',
                                        'orderCPF'      => null,
                                        'payments'      => array(),
                                        'changeValue'   => 0,
                                        'toTravel'      => false,
                                        'isScheduled'   => !empty($lastSale['DTHRAGENDADA']),
                                        'scheduledDate' => $lastSale['DTHRAGENDADA'],
                                        'deliveryAddress' => array(
                                            'streetAddress' => $streetAddress,
                                            'neighborhood'  => $lastSale['DSBAIRRO'],
                                            'number'        => $number,
                                            'addressLine2'  => $lastSale['DSCOMPLENDCOCOM'],
                                            'reference'     => $lastSale['DSREFENDCONSCOM'],
                                            'city'          => $lastSale['CDMUNICIBGE'],
                                            'state'         => $lastSale['SGESTADO'],
                                            'country'       => $lastSale['CDPAIS'],
                                            'zipCode'       => $lastSale['NRCEPCONSCOMAND'],
                                            'phone'         => null,
                                            'cellphone'     => null 
                                        )
                                    )
                                );
                                if(isset($lastSaleData['orderDescription']['toTravel']))
                                    $lastSaleData['orderDescription']['toTravel'] = $this->setToTravel($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], null);
                                // Busca pelos produtos relacionados à venda
                                $return = $this->consumerService->getLastSaleProducts($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $clientCode, $consumerCode, $isComandaVen);
                                if (!$return['error']) {
                                    $products = $return['data'];
                                    // Formata dados retornados para cada item da venda
                                    foreach ($products as $product) {
                                        if(!empty($product['NRINSCRCONS'])){
                                            $lastSaleData['orderDescription']['orderCPF'] = $product['NRINSCRCONS'];
                                        }
                                        $unitValue = $this->util->removeDecimal($product['VRPRECCOMVEN']);
                                        $discount  = $this->util->removeDecimal($product['VRDESCCOMVEN']);
                                        $quantity  = $this->util->float($product['QTPRODCOMVEN']);
                                        $itcomandavenSeq = isset($product['NRPRODCOMVEN']) ? $product['NRPRODCOMVEN'] : null;

                                        $lastSaleData['orderDescription']['productsValue'] += $unitValue*$quantity - $discount; 

                                        $productData = array(
                                            'productCode'        => $product['CDPRODUTO'],
                                            'productName'        => $product['NMPRODUTO'],
                                            'quantity'           => $quantity,
                                            'unitValue'          => $unitValue,
                                            'discount'           => $discount,
                                            'observations'       => array()
                                        );
                                        $return = $this->consumerService->getLastSaleProductObservations($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $itcomandavenSeq, $isComandaVen);
                                        if(!$return['error']){
                                            $observations = $return['data'];
                                            foreach($observations as $observation){
                                                $observationData = array(
                                                    'obsGroupCode' => $observation['CDGRPOCOR'],
                                                    'obsCode'      => $observation['CDOCORR'],
                                                    'obsName'      => $observation['DSOCORR'],
                                                    'quantity'     => 1,    
                                                    'price'        => 0    
                                                );
                                                
                                                array_push($productData['observations'], $observationData);
                                            }
                                        }
                                        array_push($lastSaleData['items'], $productData);
                                    }
                                }
                                // Busca pelos produtos combo relacionados à venda
                                $return = $this->consumerService->getLastSaleCombos($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $clientCode, $consumerCode, $isComandaVen);
                                
                                if (!$return['error']) {
                                    $combos = $return['data'];
                                    foreach ($combos as $combo) {
                                        $comboData = array(
                                            'productCode'   => $combo['CDPRODPROMOCAO'],
                                            'productName'   => $combo['NMPRODUTO'],
                                            'quantity'      => 1, 
                                            'unitValue'     => $this->util->removeDecimal($combo['VRPRECCOMVEN']),
                                            'discount'      => $this->util->removeDecimal($combo['VRDESCCOMVEN']),
                                            'observations'  => array(),
                                            'childItems'    => array()
                                        );
                                        
                                        $return = $this->consumerService->getLastSaleComboProducts($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $combo['CDPRODPROMOCAO'], $combo['NRSEQPRODCOM'], $clientCode, $consumerCode, $isComandaVen);
                                        if (!$return['error']) {
                                            $comboProducts = $return['data'];
                                            
                                            $comboName = array();
                                            $comboDescription = array();
                                            
                                            foreach ($comboProducts as $comboProduct) {
                                                if(!empty($comboProduct['NRINSCRCONS'])){
                                                    $lastSaleData['orderDescription']['orderCPF'] = $comboProduct['NRINSCRCONS'];
                                                }
                                                $unitValue = $this->util->removeDecimal($comboProduct['VRPRECCOMVEN']);
                                                $discount  = $this->util->removeDecimal($comboProduct['VRDESCCOMVEN']);
                                                $quantity  = $this->util->float($comboProduct['QTPRODCOMVEN']);
                                                $itcomandavenSeq = isset($comboProduct['NRPRODCOMVEN']) ? $comboProduct['NRPRODCOMVEN'] : null;

                                                $lastSaleData['orderDescription']['productsValue'] += $unitValue*$quantity - $discount; 
                                               
                                                $comboProductData = array(
                                                    'productCode'   => $comboProduct['CDPRODUTO'],
                                                    'productName'   => $comboProduct['NMPRODUTO'],
                                                    'quantity'      => $quantity, 
                                                    'unitValue'     => $unitValue,
                                                    'discount'      => $discount,
                                                    'observations'  => array()
                                                );
                                                $return = $this->consumerService->getLastSaleProductObservations($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $itcomandavenSeq, $isComandaVen);
                                                if(!$return['error']){
                                                    $observations = $return['data'];
                                                    foreach($observations as $observation){
                                                        $observationData = array(
                                                            'obsGroupCode' => $observation['CDGRPOCOR'],
                                                            'obsCode'      => $observation['CDOCORR'],
                                                            'obsName'      => $observation['DSOCORR'],
                                                            'quantity'     => 1,    
                                                            'price'        => null     
                                                        );
                                                        array_push($comboProductData['observations'], $observationData);
                                                    }
                                                }
                                                
                                                array_push($comboData['childItems'], $comboProductData);
                                            }
                                            array_push($lastSaleData['items'], $comboData);
                                        }
                                    }
                                }
                                // Busca pelas formas de pagamento relacionadas à venda
                                $return = $this->consumerService->getLastSalePayments($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $clientCode, $consumerCode, $isComandaVen);
                                if (!$return['error']) {
                                    $payments = $return['data'];
                                    
                                    // Formata dados retornados para cada pagamento
                                    foreach ($payments as $payment) {
                                        $paymentData = array(
                                            'paymentCode' => $payment['CDTIPORECE'],
                                            'paymentName' => $payment['NMTIPORECE'],
                                            'value'       => $this->util->removeDecimal($payment['VRMOVIVEND'])
                                        );
                                        
                                        array_push($lastSaleData['orderDescription']['payments'], $paymentData);
                                    }
                                } 
                                $orderDescription = &$lastSaleData['orderDescription'];
                                $orderDescription['totalValue'] = $orderDescription['deliveryFee'] + $orderDescription['productsValue'] - $orderDescription['orderDiscount'];  
                                array_push($data,$lastSaleData);
                            } else {
                                $message = 'Nenhuma venda foi encontrada.';
                                $response->addMessage(new Message($message));
                            }
                        } else {
                            $message = $return['message'];
                            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                        }
                    }    
                }
            }
               
            $return = $this->consumerService->getSalesByPeriod($clientCode, $consumerCode, $from, $to, $counter, $checkDate, $checkCounter, $subsidiaries);
            // Verifica se houve retorno
            if (!$return['error']) {
                $sales = $return['data'];
                
                if (!empty($sales)) {
                    // Formata os dados retornados para cada venda aprovada do consumidor
                    foreach ($sales as $sale) {
                        // Formata os dados da última venda aprovada do cliente
                        $deliveryFee   = $sale['IDRETBALLOJA'] == 'S' ? 0 : $sale['VRACRVENDA'];
                        $addressAsArray = explode(' ',$sale['DSENDECONSVENDA']);
                        $streetAddress = $addressAsArray[0] !== "" ? $addressAsArray[0] : null;
                        $number = null;
                        if(count($addressAsArray) > 1){
                            $number = $addressAsArray[1];
                        }else{
                            $addressAsArray = explode(', ',$sale['DSENDECONSVENDA']);
                            $streetAddress = $addressAsArray[0] !== "" ? $addressAsArray[0] : null;
                            if(count($addressAsArray) > 1){
                               $number = $addressAsArray[1];
                            }  
                        }
                        $orderDiscount = $this->util->removeDecimal($sale['VRDESCVENDA']);
                        $saleData = array(
                            'subsidiaryCode'   => $sale['CDFILIAL'],
                            'clientCode'       => $sale['CDCLIENTE'],
                            'consumerCode'     => $sale['CDCONSUMIDOR'],
                            'saleDate'         => $sale['DTVENDA'],
                            'saleCode'         => $sale['NRCOMANDA'],
                            'orderStatus'      => self::status[$sale['IDSITUVENDA']],
                            'isOpen'           => !!$sale['ISOPEN'] ? 'true' : 'false',
                            'items'            => array(),
                            'orderDescription' => array(
                                'orderType'     => $sale['IDORIGEMVENDA'],
                                'productsValue' => 0,
                                'deliveryFee'   => $deliveryFee,
                                'orderDiscount' => $orderDiscount,
                                'totalValue'    => 0,
                                'description'   => '',
                                'orderCPF'      => $sale['NRINSCRCONS'],
                                'payments'      => array(),
                                'changeValue'   => 0,
                                'toTravel'      => false,
                                'isScheduled'   => !empty($sale['DTHRAGENDADA']),
                                'scheduledDate' => $sale['DTHRAGENDADA'],
                                'deliveryAddress' => array(
                                    'streetAddress' => $streetAddress,
                                    'neighborhood'  => $sale['DSBAIRRO'],
                                    'number'        => $number,
                                    'addressLine2'  => $sale['DSCOMPLENDCOVEN'],
                                    'reference'     => $sale['DSREFENDCONSVEN'],
                                    'city'          => $sale['CDMUNICIBGE'],
                                    'state'         => $sale['SGESTADO'],
                                    'country'       => $sale['CDPAIS'],
                                    'zipCode'       => $sale['NRCEPCONSVENDA'],
                                    'phone'         => null,
                                    'cellphone'     => null 
                                )
                            )
                        );
                        if(isset($saleData['orderDescription']['toTravel']))
                            $saleData['orderDescription']['toTravel'] = $this->setToTravel($sale['CDFILIAL'], null ,null, $sale['NRSEQVENDA']);
                        // Busca pelos produtos relacionados à venda
                        $return = $this->consumerService->getProducts($sale['CDFILIAL'], $sale['CDCAIXA'], $sale['NRSEQVENDA']);
                        if (!$return['error']) {
                            $products = $return['data'];
                            // Formata dados retornados para cada item da venda
                            foreach ($products as $product) {
                                $unitValue  = $this->util->removeDecimal($product['VRUNITVEND']);
                                $discount   = $this->util->removeDecimal($product['VRDESITVEND']);
                                $addition   = $this->util->removeDecimal($product['VRACRITVEND']);
                                $quantity   = $this->util->float($product['QTPRODVEND']);
                                $productSeq = $product['NRSEQUITVEND'];

                                $saleData['orderDescription']['productsValue'] += $unitValue*$quantity - $discount + $addition; 

                                $productData = array(
                                    'productCode'        => $product['CDPRODUTO'],
                                    'productName'        => $product['NMPRODUTO'],
                                    'quantity'           => $quantity,
                                    'unitValue'          => $unitValue,
                                    'discount'           => $discount,
                                    'observations'       => array()
                                );
                                
                                $return = $this->consumerService->getSaleProductObservations($sale['CDFILIAL'], $sale['CDCAIXA'], $sale['NRSEQVENDA'], $productSeq);
         
                                if(!$return['error']){
                                    $observations = $return['data'];
                                    foreach($observations as $observation){
                                        $observationData = array(
                                            'obsGroupCode' => $observation['CDGRPOCOR'],
                                            'obsCode'      => $observation['CDOCORR'],
                                            'obsName'      => $observation['DSOCORR'],
                                            'quantity'     => 1,    
                                            'price'        => null    
                                        );
                                        
                                        array_push($productData['observations'], $observationData);
                                    }
                                }
                                array_push($saleData['items'], $productData);
                            }
                        }
                        // Busca pelos produtos combo relacionados à venda
                        $return = $this->consumerService->getCombos($sale['CDFILIAL'], $sale['CDCAIXA'], $sale['NRSEQVENDA']);
                        if (!$return['error']) {
                            $combos = $return['data'];
                            
                            foreach ($combos as $combo) {
                                
                                $comboData = array(
                                    'productCode'   => $combo['CDPRODPROMOCAO'],
                                    'productName'   => $combo['NMPRODUTO'],
                                    'quantity'      => 1, 
                                    'unitValue'     => $this->util->removeDecimal($combo['VRUNITVEND']),
                                    'discount'      => $this->util->removeDecimal($combo['VRDESITVEND']),
                                    'observations'  => array(),
                                    'childItems'    => array()
                                );
                                // Busca pelos produtos do combo
                                $return = $this->consumerService->getComboProducts($sale['CDFILIAL'], $sale['CDCAIXA'], $sale['NRSEQVENDA'], $combo['NRSEQUITVEND'],$combo['NRSEQPRODCOMIT'], $combo['AGRUPADO']);
                                if (!$return['error']) {
                                    $comboProducts = $return['data'];
                                    
                                    $comboName = array();
                                    $comboDescription = array();
                                    
                                    foreach ($comboProducts as $comboProduct) {
                                        $unitValue  = $this->util->removeDecimal($comboProduct['VRUNITVEND']);
                                        $discount   = $this->util->removeDecimal($comboProduct['VRDESITVEND']);
                                        $addition   = $this->util->removeDecimal($comboProduct['VRACRITVEND']);
                                        $quantity   = $this->util->float($comboProduct['QTPRODVEND']);
                                        $productSeq = $comboProduct['NRSEQUITVEND'];
                                        
                                        $saleData['orderDescription']['productsValue'] += $unitValue*$quantity - $discount + $addition; 
                                        $comboProductData = array(
                                            'productCode'   => $comboProduct['CDPRODUTO'],
                                            'productName'   => $comboProduct['NMPRODUTO'],
                                            'quantity'      => $quantity, 
                                            'unitValue'     => $unitValue,
                                            'discount'      => $discount,
                                            'observations'  => array()
                                        );
                                        array_push($comboData['childItems'], $comboProductData);
                                    }
                                    if(empty($itcomandavenSeq)){
                                    $isAlreadyInsert = false;
                                    foreach($saleData['items'] as &$item){
                                        if($item['productCode'] == $combo['CDPRODPROMOCAO']){
                                            array_push($item['childItems'], $comboData['childItems'][0]);
                                            $item['unitValue'] += $comboData['unitValue'];
                                            $item['discount'] += $comboData['discount'];
                                            $isAlreadyInsert = true;
                                        }
                                    }
                                    if($isAlreadyInsert == false)
                                        array_push($saleData['items'], $comboData);
                                }else
                                    array_push($saleData['items'], $comboData);
                                }
                                else {
                                    $message = $return['message'];
                                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                }
                            }
                        } 
                        else {
                            $message = $return['message'];
                            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                        }
                        // Busca pelas formas de pagamento relacionadas à venda
                        $return = $this->consumerService->getSalePayments($sale['CDFILIAL'], $sale['CDCAIXA'], $sale['NRSEQVENDA']);
                        if (!$return['error']) {
                            $payments = $return['data'];
                            
                            // Formata dados retornados para cada pagamento
                            foreach ($payments as $payment) {
                                $paymentData = array(
                                    'paymentCode' => $payment['CDTIPORECE'],
                                    'paymentName' => $payment['NMTIPORECE'],
                                    'value'       => $this->util->removeDecimal($payment['VRMOVIVEND'])
                                );
                            
                                array_push($saleData['orderDescription']['payments'], $paymentData);
                            }
                        }
                        $orderDescription = &$saleData['orderDescription'];
                        $orderDescription['totalValue'] = $orderDescription['deliveryFee'] + $orderDescription['productsValue'] - $orderDescription['orderDiscount'];  
                        array_push($data, $saleData);
                    }
                    
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    if(count($data)>0){
                        $response->addDataSet(new DataSet('', $data));
                    }else{    
                        $message = 'Nenhuma venda foi encontrada.';
                        $response->addMessage(new Message($message));
                    }
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message), self::ERROR_MESSAGE);
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao buscar vendas por período. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    /* 
        Retorna a última venda aprovada de um consumidor. 
    */

    public function getSaleById(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // Dados a serem retornados

            $params = $input;
            $saleId         = isset($params['saleCode']) ? $params['saleCode'] : $params['saleId'];
            $subsidiaryCode = $params['subsidiaryCode'];
            $clientCode     = $params['clientCode'];
            $consumerCode   = $params['consumerCode'];  

            $return = $this->consumerService->getLastSale($saleId, $subsidiaryCode, $clientCode, $consumerCode);
            if (!$return['error']) {
                $isComandaVen = $return['isComandaVen'];
                $lastSale = $return['data'];
                
                if (!empty($lastSale)) {
                    // Formata os dados da última venda aprovada do cliente
                    $deliveryFee   = $lastSale['IDRETBALLOJA'] == 'S' ? 0 : $lastSale['VRACRCOMANDA'];
                    $addressAsArray = explode(',',$lastSale['DSENDECONSCOMAN']);
                    $streetAddress = $addressAsArray[0] !== "" ? $addressAsArray[0] : null;
                    $number = null;
                    if(count($addressAsArray) > 1){
                        $number = $addressAsArray[1];
                    }
                    
                    $orderDiscount = $this->util->removeDecimal($lastSale['VRDESCOMANDA']);
                    $lastSaleData = array(
                        'subsidiaryCode'   => $lastSale['CDFILIAL'],
                        'clientCode'       => $lastSale['CDCLIENTE'],
                        'consumerCode'     => $lastSale['CDCONSUMIDOR'],
                        'saleDate'         => $lastSale['DTVENDA'],
                        'saleCode'         => $lastSale['NRCOMANDA'],
                        'orderStatus'      => self::status[$lastSale['IDSTCOMANDA']],
                        'isOpen'           => !!$lastSale['ISOPEN'] ? 'true' : 'false',
                        'items'            => array(),
                        'orderDescription' => array(
                            'orderType'       => $lastSale['IDORIGEMVENDA'],
                            'productsValue'   => 0,
                            'deliveryFee'     => $deliveryFee,
                            'orderDiscount'   => $orderDiscount,
                            'totalValue'      => 0,
                            'description'     => '',
                            'orderCPF'        => null,
                            'payments'        => array(),
                            'changeValue'     => 0,
                            'toTravel'        => false,
                            'isScheduled'     => !empty($lastSale['DTHRAGENDADA']),
                            'scheduledDate'   => $lastSale['DTHRAGENDADA'],
                            'deliveryAddress' => array(
                                'streetAddress' => $streetAddress,
                                'neighborhood'  => $lastSale['DSBAIRRO'],
                                'number'        => $number,
                                'addressLine2'  => $lastSale['DSCOMPLENDCOCOM'],
                                'reference'     => $lastSale['DSREFENDCONSCOM'],
                                'city'          => $lastSale['CDMUNICIBGE'],
                                'state'         => $lastSale['SGESTADO'],
                                'country'       => $lastSale['CDPAIS'],
                                'zipCode'       => $lastSale['NRCEPCONSCOMAND'],
                                'phone'         => null,
                                'cellphone'     => null 
                            )
                        )
                    );

                    if(isset($lastSaleData['orderDescription']['toTravel']))
                        $lastSaleData['orderDescription']['toTravel'] = $this->setToTravel($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], null);
                    // busca pelos produtos relacionados à venda
                    $return = $this->consumerService->getLastSaleProducts($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $clientCode ,$consumerCode, $isComandaVen);
                    if (!$return['error']) {
                        $products = $return['data'];
                        // formata dados retornados para cada item da venda
                        foreach ($products as $product) {
                            if(!empty($product['NRINSCRCONS'])){
                                $lastSaleData['orderDescription']['orderCPF'] = $product['NRINSCRCONS'];
                            }
                            $unitValue = $this->util->removeDecimal($product['VRPRECCOMVEN']);
                            $discount  = $this->util->removeDecimal($product['VRDESCCOMVEN']);
                            $quantity  = $this->util->float($product['QTPRODCOMVEN']);
                            $itcomandavenSeq = isset($product['NRPRODCOMVEN']) ? $product['NRPRODCOMVEN'] : null;
                            $lastSaleData['orderDescription']['productsValue'] += $unitValue*$quantity - $discount; 
                            $productData = array(
                                'productCode'        => $product['CDPRODUTO'],
                                'productName'        => $product['NMPRODUTO'],
                                'quantity'           => $quantity,
                                'unitValue'          => $unitValue,
                                'discount'           => $discount,
                                'observations'       => array()
                            );
                            
                            $return = $this->consumerService->getLastSaleProductObservations($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $itcomandavenSeq, $isComandaVen);
                            if(!$return['error']){
                                $observations = $return['data'];
                                foreach($observations as $observation){
                                    $observationData = array(
                                        'obsGroupCode' => $observation['CDGRPOCOR'],
                                        'obsCode'      => $observation['CDOCORR'],
                                        'obsName'      => $observation['DSOCORR'],
                                        'quantity'     => 1,    
                                        'price'        => null    
                                    );
                                    
                                    array_push($productData['observations'], $observationData);
                                }
                            }
                            array_push($lastSaleData['items'], $productData);
                        }
                    }
                    // Busca pelos produtos combo relacionados à venda
                    $return = $this->consumerService->getLastSaleCombos($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $clientCode, $consumerCode, $isComandaVen);
                    
                    if (!$return['error']) {
                        $combos = $return['data'];
                        foreach ($combos as $combo) {
                            $comboData = array(
                                'productCode'   => $combo['CDPRODPROMOCAO'],
                                'productName'   => $combo['NMPRODUTO'],
                                'quantity'      => 1, 
                                'unitValue'     => $this->util->removeDecimal($combo['VRPRECCOMVEN']),
                                'discount'      => $this->util->removeDecimal($combo['VRDESCCOMVEN']),
                                'observations'  => array(),
                                'childItems'    => array()
                            );
                            
                            $return = $this->consumerService->getLastSaleComboProducts($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $combo['CDPRODPROMOCAO'], $combo['NRSEQPRODCOM'], $clientCode, $consumerCode, $isComandaVen);
                            if (!$return['error']) {
                                $comboProducts = $return['data'];
                                
                                $comboName = array();
                                $comboDescription = array();
                                
                                foreach ($comboProducts as $comboProduct) {
                                    if(!empty($comboProduct['NRINSCRCONS'])){
                                        $lastSaleData['orderDescription']['orderCPF'] = $comboProduct['NRINSCRCONS'];
                                    }
                                    $unitValue = $this->util->removeDecimal($comboProduct['VRPRECCOMVEN']);
                                    $discount  = $this->util->removeDecimal($comboProduct['VRDESCCOMVEN']);
                                    $quantity  = $this->util->float($comboProduct['QTPRODCOMVEN']);
                                    $itcomandavenSeq = isset($comboProduct['NRPRODCOMVEN']) ? $comboProduct['NRPRODCOMVEN'] : null;
                                    $lastSaleData['orderDescription']['productsValue'] += $unitValue*$quantity - $discount; 
                                   
                                    $comboProductData = array(
                                        'productCode'   => $comboProduct['CDPRODUTO'],
                                        'productName'   => $comboProduct['NMPRODUTO'],
                                        'quantity'      => $quantity, 
                                        'unitValue'     => $unitValue,
                                        'discount'      => $discount,
                                        'observations'  => array()
                                    );
                                    $return = $this->consumerService->getLastSaleProductObservations($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $itcomandavenSeq, $isComandaVen);
                                    if(!$return['error']){
                                        $observations = $return['data'];
                                        foreach($observations as $observation){
                                            $observationData = array(
                                                'obsGroupCode' => $observation['CDGRPOCOR'],
                                                'obsCode'      => $observation['CDOCORR'],
                                                'obsName'      => $observation['DSOCORR'],
                                                'quantity'     => 1,    
                                                'price'        => null     
                                            );
                                            array_push($comboProductData['observations'], $observationData);
                                        }
                                    }
                                    
                                    array_push($comboData['childItems'], $comboProductData);
                                }
                                if(empty($itcomandavenSeq)){
                                    $isAlreadyInsert = false;
                                    foreach($lastSaleData['items'] as &$item){
                                        if($item['productCode'] == $combo['CDPRODPROMOCAO']){
                                            array_push($item['childItems'], $comboData['childItems'][0]);
                                            $item['unitValue'] += $comboData['unitValue'];
                                            $item['discount'] += $comboData['discount'];
                                            $isAlreadyInsert = true;
                                        }
                                    }
                                    if($isAlreadyInsert == false)
                                        array_push($lastSaleData['items'], $comboData);
                                }else
                                    array_push($lastSaleData['items'], $comboData);
                            }
                        }
                    }
                    // Busca pelas formas de pagamento relacionadas à venda
                    $return = $this->consumerService->getLastSalePayments($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $clientCode, $consumerCode, $isComandaVen);
                    if (!$return['error']) {
                        $payments = $return['data'];
                        
                        // Formata dados retornados para cada pagamento
                        foreach ($payments as $payment) {
                            $paymentData = array(
                                'paymentCode' => $payment['CDTIPORECE'],
                                'paymentName' => $payment['NMTIPORECE'],
                                'value'       => $this->util->removeDecimal($payment['VRMOVIVEND'])
                            );
                            array_push($lastSaleData['orderDescription']['payments'], $paymentData);
                        }
                    } 
                    $orderDescription = &$lastSaleData['orderDescription'];
                    $orderDescription['totalValue'] = $orderDescription['deliveryFee'] + $orderDescription['productsValue'] - $orderDescription['orderDiscount'];  
                    
                    $data = array($lastSaleData);
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    $message = 'Nenhuma venda foi encontrada.';
                    $response->addMessage(new Message($message));
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao buscar última venda do consumidor. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    /* 
        Retorna o extrato de vendas de um consumidor no período especificado. 
    */
    public function getSalesExtractByPeriod(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (empty($input)) {
                throw new \Exception(utf8_decode('Não foram recebidos parâmetros'));
            }
            
            $data   = array(); // Dados a serem retornados

            $params       = $input;
            $clientCode   = $params['clientCode'];
            $consumerCode = $params['consumerCode'];
            $from         = $params['from'];
            $to           = $params['to'];
            
            // Verifica se existe cliente cadastrado com essas informações
            if (!$this->util->consumerExists($clientCode, $consumerCode)) {
                throw new \Exception('Consumidor inexistente.');
            }
            
            // Verifica se as datas foram enviadas no formato requerido
            if (!($this->util->isValidDate($from) && $this->util->isValidDate($to))) {
                throw new \Exception('Formato inválido de datas');
            }
            
            // Verifica o intervalo de tempo
            if (!($this->util->isValidDateInterval($from, $to))) {
                throw new \Exception('Intervalo de datas inválido.');
            }
            
            $return = $this->consumerService->getSalesExtractByPeriod($clientCode, $consumerCode, $from, $to);
            
            if (!$return['error']) {
                $salesExtract = $return['data'];
                if (!empty($salesExtract)) {
                    // Formata os dados retornados para cada extrato de venda
                    foreach ($salesExtract as $extract) {
                        $extractData = array(
                            'clientCode'      => $extract['CDCLIENTE'],
                            'consumerCode'    => $extract['CDCONSUMIDOR'],
                            'subsidiaryCode'  => $extract['CDFILIAL'],
                            'saleDate'        => $extract['DTVENDA'],
                            'saleValue'       => $this->util->removeDecimal($extract['VRTOTALVENDA']),
                            'consumerBalance' => $this->util->removeDecimal($extract['VRSALDCONEXT'])
                        );
                        
                        array_push($data, $extractData);
                    }
                    
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    $message = 'Não foram encontradas informações de extrato do consumidor.';
                    $response->addMessage(new Message($message));
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao buscar extrato do consumidor. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function hasOpenSale(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $this->util->beginTransaction();
        try {
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // Dados a serem retornados

            $params        = $input;
            $clientCode    = $params['clientCode'];
            $consumerCode  = $params['consumerCode'];
            $subsidiaries  = !empty($params['subsidiaries']) ? $params['subsidiaries'] : $this->util->getSubsidiaries();
            // Verifica se existe cliente cadastrado com essas informações
            if (!$this->util->consumerExists($clientCode, $consumerCode)) {
                throw new \Exception('Consumidor inexistente.');
            }
            
            $return = $this->consumerService->hasOpenSale($consumerCode, $clientCode, $subsidiaries);

            if (!$return['error']) {
                $openSales = $return['data'];
                $data  = array(
                    'hasOpenSale' => count($openSales) > 0
                );
                $response->addDataSet(new DataSet('', array($data)));
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao buscar vendas abertas do consumidor. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function doCheckin(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true); 
        $this->util->beginTransaction();
        try {
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // Dados a serem retornados
            $params = $input;

            $checkinId    = $params['checkinId'];
            $clientCode   = $params['clientCode'];
            $consumerCode = $params['consumerCode'];

            // Verifica se existe cliente cadastrado com essas informações
            if (!$this->util->consumerExists($clientCode, $consumerCode)) {
                throw new \Exception('Consumidor inexistente.');
            }

            $return = $this->consumerService->doCheckin($checkinId, $clientCode, $consumerCode);
            $data   =  array();
            if(!$return['error']){
                $checkin = $return['data'];
                $checkinData = array(
                    'positionNumber' => $checkin['NRPOSICAO'],
                    'tableNumber'    => (int)$checkin['NRMESA']    
                );
                array_push($data,$checkinData);
                $response->addDataSet(new DataSet('', $data));
            }else{
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        } catch (\Exception $e) {
            $this->util->rollBack();
            $message = 'Erro ao realizar checkin. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    private function getConsumerItemPrice($subsidiaryCode,$clientCode,$productCode,$storeCode,$consumerCode){
        
        $return = array(
            'newPrice'      => 0,
            'discountType'  => '',
            'discountValue' => 0
        );
        $priceDetails = $this->orderService->recalculateItensPrice($subsidiaryCode,$clientCode,$productCode,$storeCode,$consumerCode);
        if(!$priceDetails['error']){
            $return['newPrice']      = ($priceDetails['PRECO'] + $priceDetails['ACRE']) - $priceDetails['DESC'];
            $return['discountType']  = $priceDetails['TIPO'];
            $return['discountValue'] = $priceDetails['PRECODIA'];
        }
        return $return;
    }
    
    private function getNewDiscount($priceDetails,$item){

        $discountType = $priceDetails['discountType'];
        $discountValue = $priceDetails['discountValue'];
        
        $itemDiscount = isset($item['VRDESCCOMVEN']) ? $item['VRDESCCOMVEN'] : $item['discount'];

        if($discountType == 'V')
            $discount = $itemDiscount + $itemDiscount - $discountValue;
        else
            $discount = $itemDiscount - ($itemDiscount * ($discountValue/100));

        return self::numberFormatPrecision($discount);
    }

    private function numberFormatPrecision($number, $precision = 2, $separator = '.')
    {
        $numberParts = explode($separator, $number);
        $response = $numberParts[0];
        if(count($numberParts)>1){
            $response .= $separator;
            $response .= substr($numberParts[1], 0, $precision);
        }
        return $response;
    }

    public function getTableBill(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Consumer','getTableBill');
        $input = json_decode($inputJSON, true); 
        $this->util->beginTransaction();
        try {
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // Dados a serem retornados
            
            $params    = $input;
            $checkinId    = $params['checkinId'];
            $clientCode   = isset($params['clientCode']) ? $params['clientCode'] : null;
            $consumerCode = isset($params['consumerCode']) ? $params['consumerCode'] : null;
            
            $validCheckinId = $this->orderService->isValidCheckinId($checkinId);
            if($validCheckinId['error']){
                throw new \Exception($validCheckinId['message']);
            }

            $resultCheckinMesPos = $this->orderService->getResultCheckinMesPos($checkinId);
            if($resultCheckinMesPos['error']){
                throw new \Exception($resultCheckinMesPos['message']);
            }
            $checkinMesPos   = $resultCheckinMesPos['data'];
            $subsidiaryCode  = $checkinMesPos['CDFILIAL'];
            $vendarestSeq    = $checkinMesPos['NRVENDAREST']; 
            $comandavenSeq   = $checkinMesPos['NRCOMANDA'];
            $storeCode       = $checkinMesPos['CDLOJA'];
            $storeServiceTax = $this->util->getStoreServiceTax($subsidiaryCode,$storeCode);
            $return = $this->consumerService->getTableBill($subsidiaryCode,$vendarestSeq,$comandavenSeq);
            if (!$return['error']) {
                $positions = $return['data'];
                if (!empty($positions)) {
                    foreach ($positions as $lastSale) {
                        // Formata os dados da última venda aprovada do cliente
                        $deliveryFee   = 0;
                        $addressAsArray = explode(',',$lastSale['DSENDECONSCOMAN']);
                        $streetAddress = $addressAsArray[0] !== "" ? $addressAsArray[0] : null;
                        $number = null;
                        if(count($addressAsArray) > 1){
                            $number = $addressAsArray[1];
                        }
     
                        $orderDiscount  = $this->util->removeDecimal($lastSale['VRDESCOMANDA']);
                        $tableStatus    = $this->util->getTableStatus($subsidiaryCode,$storeCode,$lastSale['NRMESA']);
                        $positionStatus = $this->util->getPositionStatus($subsidiaryCode,$vendarestSeq, $comandavenSeq, $lastSale['NRPOSICAO']);
                        if(($tableStatus == 'S' || $tableStatus == 'P') && $positionStatus != 'P'){
                            $positionStatus = $tableStatus;
                        }
                        $lastSaleData = array(
                            'subsidiaryCode'   => $lastSale['CDFILIAL'],
                            'clientCode'       => $lastSale['CDCLIENTE'],
                            'consumerCode'     => $lastSale['CDCONSUMIDOR'],
                            'positionNumber'   => $lastSale['NRPOSICAO'],
                            'checkinId'        => $lastSale['IDCHECKIN'],
                            'consumerName'     => $lastSale['NMCONSUMIDOR'],
                            'tableNumber'      => (int)$lastSale['NRMESA'],     
                            'saleDate'         => $lastSale['DTVENDA'],
                            'saleCode'         => $lastSale['NRCOMANDA'],
                            'storeCode'        => $storeCode,
                            'orderStatus'      => self::status[$lastSale['IDSTCOMANDA']],
                            'tableStatus'      => $tableStatus,
                            'positionStatus'   => $positionStatus,
                            'isOpen'           => !!$lastSale['ISOPEN'] ? 'true' : 'false',
                            'items'            => array(),
                            'orderDescription' => array(
                                'orderType'       => 'MES',
                                'productsValue'   => 0,
                                'deliveryFee'     => $deliveryFee,
                                'orderDiscount'   => $orderDiscount,
                                'serviceTax'      => 0,
                                'totalValue'      => 0,
                                'description'     => '',
                                'orderCPF'        => null,
                                'payments'        => array(),
                                'changeValue'     => 0,
                                'toTravel'        => false,
                                'isScheduled'     => !empty($lastSale['DTHRAGENDADA']),
                                'scheduledDate'   => $lastSale['DTHRAGENDADA'],
                                'deliveryAddress' => array(
                                    'streetAddress' => $streetAddress,
                                    'neighborhood'  => $lastSale['DSBAIRRO'],
                                    'number'        => $number,
                                    'addressLine2'  => $lastSale['DSCOMPLENDCOCOM'],
                                    'reference'     => $lastSale['DSREFENDCONSCOM'],
                                    'city'          => $lastSale['CDMUNICIBGE'],
                                    'state'         => $lastSale['SGESTADO'],
                                    'country'       => $lastSale['CDPAIS'],
                                    'zipCode'       => $lastSale['NRCEPCONSCOMAND'],
                                    'phone'         => null,
                                    'cellphone'     => null 
                                )
                            )
                        );
                        if(isset($lastSaleData['orderDescription']['toTravel']))
                            $lastSaleData['orderDescription']['toTravel'] = $this->setToTravel($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], null);
                        // Busca pelos produtos relacionados à venda
                        $return = $this->consumerService->getPositionProducts($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $lastSale['NRPOSICAO']);
                        if (!$return['error']) {
                            $products = $return['data'];
                            // Formata dados retornados para cada item da venda
                            foreach ($products as $product) {
                                if(!empty($product['NRINSCRCONS'])){
                                    $lastSaleData['orderDescription']['orderCPF'] = $product['NRINSCRCONS'];
                                }
                                $unitValue = $this->util->removeDecimal($product['VRPRECCOMVEN']);
                                $discount  = $this->util->removeDecimal($product['VRDESCCOMVEN']);
                                $quantity  = $this->util->float($product['QTPRODCOMVEN']);
                                
                                if(!empty($clientCode) && !empty($consumerCode)){
                                    
                                    $productCode = $product['CDPRODUTO'];
                                    $priceDetails = self::getConsumerItemPrice($subsidiaryCode,$clientCode,$productCode,$storeCode,$consumerCode);
                                    $unitValue = !empty($priceDetails['newPrice']) ? $priceDetails['newPrice'] : $product['VRPRECCOMVEN'];
                                    $unitValue = bcdiv($unitValue, 1, 2);
                                    if($unitValue != $this->util->removeDecimal($product['VRPRECCOMVEN']))
                                        $discount = self::getNewDiscount($priceDetails,$product);
                                }
                                
                                $itcomandavenSeq = isset($product['NRPRODCOMVEN']) ? $product['NRPRODCOMVEN'] : null;
                                $lastSaleData['orderDescription']['productsValue'] += $unitValue*$quantity - $discount; 
                                
                                $productData = array(
                                    'productCode'        => $product['CDPRODUTO'],
                                    'productName'        => $product['NMPRODUTO'],
                                    'quantity'           => $quantity,
                                    'unitValue'          => $unitValue,
                                    'discount'           => $discount,
                                    'observations'       => array()
                                );
                                
                                $return = $this->consumerService->getLastSaleProductObservations($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'],$itcomandavenSeq, true);
                                if(!$return['error']){
                                    $observations = $return['data'];
                                    foreach($observations as $observation){
                                        $observationData = array(
                                            'obsGroupCode' => $observation['CDGRPOCOR'],
                                            'obsCode'      => $observation['CDOCORR'],
                                            'obsName'      => $observation['DSOCORR'],
                                            'quantity'     => 1,    
                                            'price'        => null    
                                        ); 
                                        array_push($productData['observations'], $observationData);
                                    }
                                }
                                array_push($lastSaleData['items'], $productData);
                            }
                        } 
                        // Busca pelos produtos combo relacionados à venda
                        $return = $this->consumerService->getPositionCombos($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'],$lastSale['NRPOSICAO']);
                        
                        if (!$return['error']) {
                            $combos = $return['data'];
                            foreach ($combos as $combo) {
                                $comboData = array(
                                    'productCode'   => $combo['CDPRODPROMOCAO'],
                                    'productName'   => $combo['NMPRODUTO'],
                                    'quantity'      => 1, 
                                    'unitValue'     => $this->util->removeDecimal($combo['VRPRECCOMVEN']),
                                    'discount'      => $this->util->removeDecimal($combo['VRDESCCOMVEN']),
                                    'observations'  => array(),
                                    'childItems'    => array()
                                );
                                
                                $return = $this->consumerService->getPositionCombosProducts($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'],$combo['CDPRODPROMOCAO'], $combo['NRSEQPRODCOM']);
                                
                                if (!$return['error']) {
                                    $comboProducts = $return['data'];
                                    
                                    $comboName = array();
                                    $comboDescription = array();
                                    if(!empty($clientCode) && !empty($consumerCode)){
                                        $comboData['unitValue'] = 0;
                                        $comboData['discount'] = 0;
                                    }
                                    foreach ($comboProducts as $comboProduct) {
                                        if(!empty($comboProduct['NRINSCRCONS'])){
                                            $lastSaleData['orderDescription']['orderCPF'] = $comboProduct['NRINSCRCONS'];
                                        }
                                        $unitValue = $this->util->removeDecimal($comboProduct['VRPRECCOMVEN']);
                                        $discount  = $this->util->removeDecimal($comboProduct['VRDESCCOMVEN']);
                                        $quantity  = $this->util->float($comboProduct['QTPRODCOMVEN']);
                                        
                                        if(!empty($clientCode) && !empty($consumerCode)){
                                            $productCode = $comboProduct['CDPRODUTO'];
                                            $priceDetails = self::getConsumerItemPrice($subsidiaryCode,$clientCode,$productCode,$storeCode,$consumerCode);
                                            $unitValue = !empty($priceDetails['newPrice']) ? $priceDetails['newPrice'] : $comboProduct['VRPRECCOMVEN'];
                                            $unitValue = bcdiv($unitValue, 1, 2);
                                            if($unitValue != $this->util->removeDecimal($comboProduct['VRPRECCOMVEN']))
                                                $discount = self::getNewDiscount($priceDetails,$comboProduct);
                                            $comboData['unitValue'] += $unitValue;
                                            $comboData['discount'] += $discount;
                                            
                                        }
                                        $itcomandavenSeq = isset($comboProduct['NRPRODCOMVEN']) ?  $comboProduct['NRPRODCOMVEN'] : null;
                                        $lastSaleData['orderDescription']['productsValue'] += $unitValue*$quantity - $discount; 
                                       
                                        $comboProductData = array(
                                            'productCode'   => $comboProduct['CDPRODUTO'],
                                            'productName'   => $comboProduct['NMPRODUTO'],
                                            'quantity'      => $quantity, 
                                            'unitValue'     => $unitValue,
                                            'discount'      => $discount,
                                            'observations'  => array()
                                        );
                                        
                                        $return = $this->consumerService->getLastSaleProductObservations($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $itcomandavenSeq, true);
                                        if(!$return['error']){
                                            $observations = $return['data'];
                                            foreach($observations as $observation){
                                                $observationData = array(
                                                    'obsGroupCode' => $observation['CDGRPOCOR'],
                                                    'obsCode'      => $observation['CDOCORR'],
                                                    'obsName'      => $observation['DSOCORR'],
                                                    'quantity'     => 1,    
                                                    'price'        => null     
                                                );
                                                array_push($comboProductData['observations'], $observationData);
                                            }
                                        } 
                                        array_push($comboData['childItems'], $comboProductData);
                                    }
                                    array_push($lastSaleData['items'], $comboData);
                                } 
                            }
                        }
                        $orderDescription = &$lastSaleData['orderDescription'];
                        $orderDescription['totalValue'] = $orderDescription['deliveryFee'] + $orderDescription['productsValue'] - $orderDescription['orderDiscount'];  
                        if(!empty($storeServiceTax)){
                            $usesCourvert     = $storeServiceTax['IDCOUVERART'] == 'S';
                            $courvertProduct  = $storeServiceTax['CDPRODCOUVER'];
                            $usesServiceTax   = $storeServiceTax['IDCOMISVENDA'] == 'S';
                            $servTaxAsProduct = $storeServiceTax['IDTRATTAXASERV'] == 'P';
                            $servTaxProduct   = $storeServiceTax['CDPRODTAXASERV'];
                            $servTaxValue     = $storeServiceTax['VRCOMISVENDA'];
                            if(($usesServiceTax) && (!empty($servTaxValue)) && (($servTaxAsProduct && !empty($servTaxProduct)) || (!$servTaxAsProduct))){
                                $orderDescription['serviceTax'] = $this->util->removeDecimal($orderDescription['totalValue']/(float)$servTaxValue);
                            }
                        }
                        array_push($data,$lastSaleData);
                    }                       
                    
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    $message = 'Nenhuma venda foi encontrada.';
                    $response->addMessage(new Message($message));
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
            $this->util->commit();
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao buscar conta da mesa. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    private function setToTravel($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRSEQVENDA){
        $toTravelObservation = $this->util->getToTravelObservation();
        $CDGRPOCOR = $toTravelObservation['CDGRPOCOR'];
        $CDOCORR   = $toTravelObservation['CDOCORR'];
        return $this->util->getTotravelObservationOrder($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRSEQVENDA, $CDGRPOCOR, $CDOCORR);
    }
    
    public function getFidelityBalance(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Consumer','getFidelityBalance');
        $input = json_decode($inputJSON, true); 
        try {
            if (empty($input)) throw new \Exception('Não foram recebidos parâmetros');
            
            $data = array(); // Dados a serem retornados
            $params = $input;
            
            $cdcliente = $params['clientCode'];
            $cdconsumidor = $params['consumerCode'];
            $consumerBalance = $this->util->getConsumerFidelity($cdcliente, $cdconsumidor);
            $orderFidelityBalance = $this->util->getValueFidelityOrders($cdcliente, $cdconsumidor);
            $consumerBalance['value'] = $this->util->subtract($consumerBalance['value'], $orderFidelityBalance);
            if(empty($consumerBalance)) $response->addMessage(new Message("Não foi encontrado saldo para este consumidor."));
            else $response->addDataSet(new DataSet('', array($consumerBalance)));
            
            
        }catch(\Exception $e){
            $message = 'Erro ao buscar saldo fidelidade do cliente. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    public function fidelityBalance(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Consumer','fidelityBalance');
        $input = json_decode($inputJSON, true); 
        try {
            if (empty($input)) throw new \Exception('Não foram recebidos parâmetros');
            
            $data = array(); // Dados a serem retornados
            $params = $input;
            
            if($params['clientCode'] !== null && $params['clientCode'] === "") throw new \Exception("Parâmetro clientCode inválido");
            if($params['consumerCode'] !== null && $params['consumerCode'] === "") throw new \Exception("Parâmetro consumerCode inválido");
            // if($params['cdfamilisald'] !== null && $params['cdfamilisald'] === "") throw new \Exception("Parâmetro cdfamilisald inválido");
            if($params['description'] !== null && $params['description'] === "") throw new \Exception("Parâmetro description inválido");
            if($params['value'] !== null && $params['value'] === "") throw new \Exception("Parâmetro value inválido");
            if($params['operationType'] !== null && $params['operationType'] === "" && ($params['operationType'] !== 'F' || $params['operationType'] !== 'E')) throw new \Exception("Parâmetro operationType inválido");
            $date = !empty($params['date']) ? $params['date'] : date('d/m/Y H:i:s');
            
            $this->validateDateInput($date);
            
            $cdcliente = $params['clientCode'];
            $cdconsumidor = $params['consumerCode'];
            // $cdfamilisald = $params['cdfamilisald'];
            $cdfamilisald = '010';
            $description = $params['description'];
            $value = $params['value'];
            $operationType = $params['operationType'];
            
            $this->saveExtratocons($cdcliente, $cdconsumidor, $cdfamilisald, $description, $value, $operationType, $date);
            
            $familyValue = $this->util->getBalanceValue($cdcliente, $cdconsumidor, $cdfamilisald);
            
            $response->addDataSet(new DataSet('', array('total' => $familyValue)));
            
        }catch(\Exception $e){
            $message = 'Erro ao gerenciar saldo fidelidade. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    private function validateDateInput($date){
        $isValid = \DateTime::CreateFromFormat('d/m/Y H:i:s', $date);
        
        if($isValid === false) throw new \Exception("Data informada com formato errado, formato esperado 'd/m/Y H:i:s'");
    }
    
    private function saveExtratocons($cdcliente, $cdconsumidor, $cdfamilisald, $description, $value, $operationType, $date){
        $extratoCons = new \SimpleXMLElement('<ExtratoCons/>');
        $xmlFields = $this->getXMLFields();
        $registro = $extratoCons->addChild('Registro');
        foreach ($xmlFields as $field) 
            $registro->addChild($field);
        $registro->cdcliente    = $cdcliente;
        $registro->cdconsumidor = $cdconsumidor;
        $registro->cdfamilisald = $cdfamilisald;
        $registro->dtmovextcons = $date;
        $registro->nrseqmovext  = '001';
        $registro->cdtiporece   = '001';
        $registro->dsopeextcons = $this->removeAcentos($description);
        $registro->vrmovextcons = $value;
        $registro->vrsaldconext = 0;
        $registro->idtpmovext   = $operationType;
        $registro->idimpextrato = 'S';
        
        $xmlToSend = $extratoCons->asXML();
        
        $params = array(
            'P_XMLEXTCONS' => $xmlToSend    
        );
        $this->util->callExtratocons($params, 'T', "InsereDados");
    }
    
    private function removeAcentos($texto){
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/", "/(¨|&|<|>)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
    
    private function getXMLFields(){
        return array("cdcliente","cdconsumidor","cdfamilisald","dtmovextcons","nrseqmovext","cdtiporece","cdfilial","cdcaixa","dsopeextcons","vrmovextcons","vrsaldconext","idtpmovext","nrdeposicons","nrseqmovcaixa","idimpextrato");
    }
    
    public function getSales(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $this->util->saveLog($inputJSON,'Consumer','getSales');
        $input = json_decode($inputJSON, true); 
        try {
            if (empty($input)) throw new \Exception('Não foram recebidos parâmetros');
            
            $data = array(); // Dados a serem retornados
            $params = $input;
            if($params['initialDate'] !== null && $params['initialDate'] === "") throw new \Exception("Parâmetro initialDate inválido");
            if($params['finalDate'] !== null && $params['finalDate'] === "") throw new \Exception("Parâmetro finalDate inválido");
            $initialDate = $params['initialDate'];
            $finalDate = $params['finalDate'];
            $cpf = isset($params['cpf']) ? $params['cpf'] : 'T';
            
            $sales = $this->getSalesInfo($initialDate, $finalDate, $cpf);
            
            $response->addDataSet(new DataSet('', $sales));
            
        }catch(\Exception $e){
            $message = 'Erro ao recuperar vendas. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    private function getSalesInfo($initialDate, $finalDate, $cpf){
        $salesData = array();
        $sales = $this->util->getSalesInfo($initialDate, $finalDate, $cpf);
        foreach ($sales as $sale) {
            $saleData = array();
            $nrseqvenda = $sale['NRSEQVENDA'];
            $cdcaixa = $sale['CDCAIXA'];
            $cdfilial = $sale['CDFILIAL'];
            $valuesInfo = $this->getValuesInfo($cdfilial, $cdcaixa, $nrseqvenda);
            $saleData = array(
                'subsidiaryCode' => $sale['CDFILIAL'],
                'clientCode' => $sale['CDCLIENTE'],
                'consumerCode' => $sale['CDCONSUMIDOR'],
                'storeCode' => $sale['CDLOJA'],
                'saleDate' => $sale['DTVENDA'],
                'saleCode' => $sale['NRCOMANDA'],
                'items' => $this->getItems($cdfilial, $nrseqvenda, $cdcaixa),
                'orderDescription' => array(
                    'orderType' => $sale['IDORIGEMVENDA'],
                    'productsValue' => $this->util->removeDecimal($valuesInfo['VALORITEMS']),
                    'deliveryFee' => $this->util->removeDecimal($sale['VRACRCOMANDA']),
                    'orderDiscount' => $this->util->removeDecimal($valuesInfo['DESCONTO']),
                    'totalValue' => $this->util->removeDecimal($valuesInfo['TOTAL']),
                    'description' => $sale['DSOBSCOMANDA'],
                    'sefazKey' => $sale['NRACESSONFCE'],
                    'saleCoupon' => $sale['NRNOTAFISCALCE'],
                    'orderCPF' => $sale['NRINSCRCONS'],
                    'payments' => $this->getPayments($cdfilial, $cdcaixa, $nrseqvenda),
                    // 'changeValue' => 0, VER Q Q VAI FAZER COM ISSO AQUI
                    'toTravel' => false,
                    'isScheduled' => $sale['ISSCHEDULED'],
                    'scheduledDate' => $sale['DTHRAGENDADA'],
                    'deliveryAddress' => array(
                        'streetAddress' => $sale['DSENDECONSVENDA'],
                        'neighborhood' => $sale['DSBAIRRO'],
                        'number' => $sale['NRENDECONSVENDA'],
                        'addressLine2' => $sale['DSCOMPLENDCOVEN'],
                        'reference' => $sale['DSREFENDCONSVEN'],
                        'city' => $sale['NMMUNICIPIO'],
                        'state' => $sale['SGESTADO'],
                        'zipCode' => $sale['NRCEPCONSVENDA'],
                        'phone' => $sale['NRFONECLIE'],
                        'cellphone' => $sale['NRFONECLIE']
                    )
                )
            );
            array_push($salesData, $saleData);
        }
        return $salesData;
    }
    
    private function getItems($cdfilial, $nrseqvenda, $cdcaixa){
        $data = array();
        $params = array(
            'CDFILIAL' => $cdfilial,
            'NRSEQVENDA' => $nrseqvenda,
            'CDCAIXA' => $cdcaixa
        );
        $simpleItem = $this->consumerService->getProducts($cdfilial, $cdcaixa, $nrseqvenda);
        if($simpleItem['error'] === false){
            $simpleItem = $simpleItem['data'];
            foreach ($simpleItem as $item) {
                $formatedItem = $this->getItemData($item, $cdfilial, $nrseqvenda, $cdcaixa, false);
                array_push($data,$formatedItem);
            }
        }
        $combos = $this->consumerService->getCombos($cdfilial, $cdcaixa, $nrseqvenda);
        if($combos['error'] === false){
            $combos = $combos['data'];
            foreach ($combos as $combo) {
                $formatedCombos = $this->getItemData($combo, $cdfilial, $nrseqvenda, $cdcaixa, true);
                array_push($data,$formatedCombos);
            }
        }
        return $data;
    }
    
    private function getItemData($item, $cdfilial, $nrseqvenda, $cdcaixa, $hasChild){
        $data = array();
        $observations = array();
        if(!empty($item['NRSEQUITVEND'])) $observations = $this->getObservations($cdfilial, $nrseqvenda, $cdcaixa, $item['NRSEQUITVEND']);  
        $data['productName'] = $item['NMPRODUTO'];
        $data['productCode'] = $item['CDPRODUTO'];
        $data['unitValue'] = $this->util->removeDecimal($item['VRUNITVEND']);
        $data['discount'] = $this->util->removeDecimal($item['VRDESITVEND']);
        $data['quantity'] = $this->util->removeDecimal(isset($item['QTPRODVEND']) ? $item['QTPRODVEND'] : 1);
        $data['observations'] = $observations;
        $data['childItems'] = array();
        if($hasChild){
            $formatedChilds = $this->getChild($cdfilial, $cdcaixa, $nrseqvenda, $item['NRSEQUITVEND'],$item['NRSEQPRODCOMIT'],$item['AGRUPADO']);
            $data['childItems'] = $formatedChilds;
        }
        return $data;
    }
    
    public function getObservations($cdfilial, $nrseqvenda, $cdcaixa, $nrsequitvend){
        $data = array();
        $observations = $this->consumerService->getSaleProductObservations($cdfilial, $cdcaixa, $nrseqvenda, $nrsequitvend);
        if($observations['error'] === false){
            $observations = $observations['data'];
            foreach ($observations as $observation) {
                $tempData = array();
                $tempData['obsName'] = $observation['DSOCORR'];
                $tempData['obsGroupCode'] = $observation['CDGRPOCOR'];
                $tempData['obsCode'] = $observation['CDOCORR'];
                $tempData['price'] = 0;
                $tempData['quantity'] = 1;
                array_push($data, $tempData);
            }
        }
        return $data;
    }
    
    private function getChild($cdfilial, $cdcaixa, $nrseqvenda, $nrsequitvend, $nrseqprodcomit, $agrupado){
        $childData = array();
        $childs = $this->consumerService->getComboProducts($cdfilial, $cdcaixa, $nrseqvenda, $nrsequitvend, $nrseqprodcomit, $agrupado);
        if($childs['error'] === false){
            $childs = $childs['data'];
            foreach ($childs as $child)
                array_push($childData,$this->getItemData($child, $cdfilial, $nrseqvenda, $cdcaixa, false));
        }
        return $childData;
    }
    
    private function getPayments($cdfilial, $cdcaixa, $nrseqvenda){
        $paymentData = array();
        $payments = $this->consumerService->getSalePayments($cdfilial, $cdcaixa, $nrseqvenda);
        if($payments['error'] === false){
            $payments = $payments['data'];
            foreach ($payments as $payment) {
                array_push($paymentData,$this->formatPayment($payment));
            }
        }
        return $paymentData;
    }
    
    private function formatPayment($payment){
        $data = array();
        $data['paymentCode'] = $payment['CDTIPORECE'];
        $data['paymentName'] = $payment['NMTIPORECE'];
        $data['value'] = $this->util->removeDecimal($payment['VRMOVIVEND']);
        return $data;
    }
    
    private function getValuesInfo($cdfilial, $cdcaixa, $nrseqvenda){
        return $this->util->getValuesInfo($cdfilial, $cdcaixa, $nrseqvenda);
    }

    public function version(Request $request, Response $response){
        try{
            $connection = $this->util->testConnection();
        }catch(\Exception $e){
            $connection = false;
        }
        $version = file_get_contents(__DIR__."/../../../version.json");
        $version = json_decode($version, true);
        $version['connection'] = $connection;
        $response->addDataset(new DataSet('', $version));
    }

    public function getBalance(Request $request, Response $response){
        try{
            $body = $request->getParameters();
    
            if(!isset($body['clientCode'])) throw new \Exception("Parâmetro clientCode não encontrado!");
            $cdcliente = $body['clientCode'];
            if(!isset($body['consumerCode'])) throw new \Exception("Parâmetro consumerCode não encontrado!");
            $cdconsumidor = $body['consumerCode'];

            if(isset($body['subsidiaryCode'])) $cdfilial = $body['subsidiaryCode'];
            else $cdfilial = null;
            
            $cdfamilisald = isset($body['cdfamilisald']) ? $body['cdfamilisald'] : '001';
    
            $balance = $this->util->getConsumerBalance($cdcliente, $cdconsumidor, $cdfamilisald, 10, $cdfilial);
            $balance = $this->util->removeDecimal($balance);
            
            $spentToday = $this->util->getTodaysExpenditure($cdcliente, $cdconsumidor, 10, $cdfilial);
            $spentToday = $this->util->removeDecimal($spentToday);
    
            $maxDailyValue = $this->consumerService->getConsumerDetails($cdcliente, $cdconsumidor);
            $maxDailyValue = !empty($maxDailyValue['VRMAXCREDCONS']) ? $this->util->removeDecimal($maxDailyValue['VRMAXCREDCONS']) : null;
    
            $response->addDataset(new DataSet('', array(
                'balance' => $balance,
                'spentToday' => $spentToday,
                'maxDailyValue' => $maxDailyValue
            )));

        }catch(\Exception $e){
            $message = 'Erro ao recuperar vendas. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function getOrdersHistory(Request $request, Response $response){
        try{
            $inputJSON = file_get_contents('php://input');
            //$this->util->saveLog($inputJSON,'consumer','getSalesByPeriod');
            $input = json_decode($inputJSON, true);
            if (empty($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            $data   = array(); // Dados a serem retornados
            
            $params       = $input;
            $clientCode   = $params['clientCode'];
            $consumerCode = $params['consumerCode'];
            
            $from          = $params['from'];
            $to            = $params['to'];
            $checkDate     = 'T'; 
            $counter       = $params['counter'];
            $checkCounter  = 'T';
            $subsidiaries  = !empty($params['subsidiaries']) ? $params['subsidiaries'] : $this->util->getSubsidiaries(); 
            
            if (!$this->util->consumerExists($clientCode, $consumerCode)) throw new \Exception('Consumidor inexistente.');
            
            if(!empty($from) && !empty($to)){
                if (!($this->util->isValidDate($from) && $this->util->isValidDate($to))) throw new \Exception('Formato inválido de datas');
                if (!($this->util->isValidDateInterval($from, $to))) throw new \Exception('Intervalo de datas inválido.');
            }else{
                $checkDate = 'F';
                $from = date('d/m/Y');
                $to   = date('d/m/Y');
            } 
            if(empty($counter)){
                $counter = 1;
                $checkCounter = 'F';
            }

            $openSales = $this->consumerService->hasOpenSale($consumerCode, $clientCode, $subsidiaries);

        }catch(\Exception $e){
            $message = 'Erro ao recuperar histórico. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function login(Request $request, Response $response){
        try{
            $body = $request->getParameters();
    
            if(empty($body['consumerId'])) throw new \Exception("O campo consumerId não foi informado!");
            else $cdidconsumid = $body['consumerId'];
            if(empty($body['password'])) throw new \Exception("O campo password não foi informado!");
            $password = $body['password'];
            if(empty($body['subsidiaryCode'])) throw new \Exception("O campo subsidiaryCode não foi informado!");
            $cdfilial = $body['subsidiaryCode'];
            if(empty($body['storeCode'])) throw new \Exception("O campo storeCode não foi informado!");
            $cdloja = $body['storeCode'];

            $consumerValidation = $this->consumerAPI->getCDIDCONSUMID(array('CDIDCONSUMID' => $cdidconsumid));
            
            if($consumerValidation['error']){
                throw new \Exception("Consumidor não encontrado!");
            } else $consumerValidation = $consumerValidation['consumer'][0];
            
            if(!$consumerValidation['CDSENHACONS']) {
                throw new \Exception("Consumidor sem senha cadastrada!");
            } else {
                $validate = $this->consumerAPI->decryptDelphi($consumerValidation['CDSENHACONS']);
                if($password == md5($validate)){
                    $consumerValidation['consumerId'] = $consumerValidation['CDIDCONSUMID'];
                    $consumerValidation['clientCode'] = $consumerValidation['CDCLIENTE'];
                    $consumerValidation['consumerCode'] = $consumerValidation['CDCONSUMIDOR'];
                    $consumerValidation['consumerName'] = $consumerValidation['NMCONSUMIDOR'];
                    unset($consumerValidation['CDIDCONSUMID']);
                    unset($consumerValidation['CDCLIENTE']);
                    unset($consumerValidation['CDCONSUMIDOR']);
                    unset($consumerValidation['NMCONSUMIDOR']);

                    $cdcliente = $consumerValidation['clientCode'];
                    $cdconsumidor = $consumerValidation['consumerCode'];

                    $typeSaleCons = $this->consumerService->getTypeSaleCons($cdcliente, $cdconsumidor);
                    $consumerValidation['typeSaleAllowed'] = $typeSaleCons['data'];
                    
                    $limitDebConsumer = $this->util->getLimitDebConsumer($cdfilial, $cdloja, $cdcliente, $cdconsumidor);
                    
                    $blockLowBalanceDebit = (isset($limitDebConsumer['VRLIMDEBCONS'])) ? $this->util->removeDecimal($limitDebConsumer['VRLIMDEBCONS']) : null;
                    $showWarningLowBalanceDebit = (isset($limitDebConsumer['VRAVIDEBCONS'])) ? $this->util->removeDecimal($limitDebConsumer['VRAVIDEBCONS']) : null;
                    $maxDailyValueDebit = (isset($limitDebConsumer['VRMAXDEBCONS'])) ? $this->util->removeDecimal($limitDebConsumer['VRMAXDEBCONS']) : null;
                    $maxMonthlyValueDebit = (isset($limitDebConsumer['VRMAXCONSMESD'])) ? $this->util->removeDecimal($limitDebConsumer['VRMAXCONSMESD']) : null;

                    $spentTodayDebit = (isset($limitDebConsumer['CONSUMO_DIA'])) ? $this->util->removeDecimal($limitDebConsumer['CONSUMO_DIA']) : null;
                    $spentMonthlyDebit = (isset($limitDebConsumer['CONSUMO_MES'])) ? $this->util->removeDecimal($limitDebConsumer['CONSUMO_MES']) : null;

                    $consumerValidation['consumerDebit']['blockLowBalance'] = $blockLowBalanceDebit;
                    $consumerValidation['consumerDebit']['showWarningLowBalance'] = $showWarningLowBalanceDebit;
                    $consumerValidation['consumerDebit']['maxDailyValue'] = $maxDailyValueDebit;
                    $consumerValidation['consumerDebit']['maxMonthlyValue'] = $maxMonthlyValueDebit;
                    $consumerValidation['consumerDebit']['spentToday'] = $spentTodayDebit;
                    $consumerValidation['consumerDebit']['spentMonth'] = $spentMonthlyDebit;
                    
                    $saldoDebConsumer = $this->util->getSaldoDebitoConsumidor($cdcliente, $cdconsumidor);
                    $saldoDebConsumer = (isset($saldoDebConsumer)) ? $this->util->removeDecimal($saldoDebConsumer) : null;

                    $consumerValidation['consumerDebit']['balance'] = $saldoDebConsumer;
                    
                    $spentToday = $this->util->getTodaysExpenditure($cdcliente, $cdconsumidor, 10, $cdfilial);
                    $spentToday = $this->util->removeDecimal($spentToday);
            
                    $consumerDetails = $this->consumerService->getConsumerDetails($cdcliente, $cdconsumidor);
                    // $maxDailyValue = !empty($consumerDetails['VRMAXCREDCONS']) ? $this->util->removeDecimal($consumerDetails['VRMAXCREDCONS']) : null;

                    $limitPersonalCredit = $this->util->getLimiteCredDisponivel($cdfilial, $cdcliente, $cdconsumidor);
                    
                    $blockLowBalanceCredit = (isset($consumerDetails['VRLIMCREDCONS'])) ? $this->util->removeDecimal($consumerDetails['VRLIMCREDCONS']) : null;
                    $showWarningLowBalanceCredit = (isset($consumerDetails['VRAVICREDCONS'])) ? $this->util->removeDecimal($consumerDetails['VRAVICREDCONS']) : null;
                    $maxDailyValueCredit = (isset($limitPersonalCredit['VRMAXCREDCONS'])) ? $this->util->removeDecimal($limitPersonalCredit['VRMAXCREDCONS']) : null;
                    
                    $consumerValidation['personalCredit']['blockLowBalance'] = $blockLowBalanceCredit;
                    $consumerValidation['personalCredit']['showWarningLowBalance'] = $showWarningLowBalanceCredit;
                    $consumerValidation['personalCredit']['maxDailyValue'] = $maxDailyValueCredit;
                    $consumerValidation['personalCredit']['spentToday'] = 0;
                    
                    $familyBalances = $this->consumerService->getFamilyBalances($cdfilial, $cdcliente, $cdconsumidor);
                    $consumerValidation['personalCredit']['familyBalances'] = $familyBalances;

                    $totalSpentToday = 0;
                    foreach($familyBalances as $family){
                        $totalSpentToday += $family['spentToday'];
                    }
                    $consumerValidation['personalCredit']['spentToday'] = $totalSpentToday;

                    $consumer = array(
                        'consumer' => $consumerValidation
                    );
                } else {
                    throw new \Exception("Senha inválida!");
                }
            }
    
            $response->addDataset(new DataSet('', $consumer));

        }catch(\Exception $e){
            $message = 'Erro ao realizar o login do consumidor. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function buyCredit(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true); 
        $this->util->beginTransaction();

        if ($this->util->databaseIsOracle())
            throw new \Exception('Carga de crédito disponível apenas em aplicacao local(MSDE).'); 

        try {
            
            if(isset($input['subsidiaryCode']))
                $cdFilial = $input['subsidiaryCode'];
            else
                throw new \Exception('Código da filial não informado.'); 

            if(isset($input['consumerId']))
                $consumer['consumerRef'] = $input['consumerId'];
            else
                throw new \Exception('Cracha do consumidor não informado.');

            if(isset($input['clientCode']))
                $consumer['clientRef'] = $input['clientCode'];
            else
                throw new \Exception('Código do cliente não informado.');

            if(isset($input['posCode']))
                $cdCaixa = $input['posCode'];
            else
                throw new \Exception('Código do caixa não informado.'); 
            
            $consumer['cpf'] = isset($input['cpf']) ? $input['cpf'] : "" ;
            $creditdData = $input['credit'];
            
            $consumerDetails = $this->consumerService->getConsumerByRef($consumer);
            
            if(empty($consumerDetails) || $consumerDetails['error'])
                throw new \Exception($consumerDetails['message']);

            $cdcliente = $consumerDetails['data']['CDCLIENTE'];
            $cdconsumidor = $consumerDetails['data']['CDCONSUMIDOR'];
            $nrOrg = $consumerDetails['data']['NRORG'];
            $dtAberCaix = $this->consumerService->getDtAbercaixa($cdFilial, $cdCaixa);
            $nrDeposiCons = date("dmyHi");
            $dtAberCaix = new \DateTime($dtAberCaix['DTABERCAIX']);         
            
            $fileName = date("Ymd-H-i-s") . "-" . $consumer['consumerRef'];
            $this->util->saveLogOrder($inputJSON, 'BuyCredit/Recebidos/' . date("m/d"), $fileName);

            $creditProcess = [];
            foreach($creditdData as $credit){
                $nrsequmovi = $this->consumerAPI->generateMovcaixaSeq($cdFilial, $cdCaixa, $dtAberCaix, $nrOrg);
                $this->consumerService->insertBuyCredit($credit, $cdFilial, $cdCaixa, $dtAberCaix, $nrOrg, $cdcliente, $nrsequmovi);
                $cdFamiliaSald = $credit['productFamilyId'];
                $tipoRece = $credit['paymentId'];
                $isFirstInsertion = $this->consumerAPI->primeiraInsercao($cdcliente, $cdconsumidor, $cdFamiliaSald);
                $currentBalance = $this->consumerAPI->buscaSaldoExtrato($cdcliente, $cdconsumidor, $cdFamiliaSald, $cdFilial);
                $valueToCredit = floatval(str_replace(',', '.', $credit['value']));
                
                // CHECK IF CONSUMER HAS BALANCE (even if it's negative).
                $balance = $this->consumerAPI->buscaSaldoCons($cdcliente, $cdconsumidor, $cdFamiliaSald);
                
                $vrRecarga = $valueToCredit;

                $this->consumerAPI->creditaSaldo($cdcliente, $cdconsumidor, $cdFamiliaSald, '001', $tipoRece, $cdFilial, $cdCaixa, $vrRecarga, $nrDeposiCons, $nrsequmovi, $dtAberCaix, "TAA - Crédito Caixa {$cdCaixa}");
                $saldoAtual = $this->consumerAPI->buscaSaldoCons($cdcliente, $cdconsumidor, $cdFamiliaSald);
                $finishcredit = [
                        'transationID'    => $nrDeposiCons,
                        'consumerId'     => $consumer['consumerRef'],
                        'consumerName'    => $consumerDetails['data']['NMCONSUMIDOR'],
                        'productFamilyId' => $credit['productFamilyId'],
                        'vrCredit'        => $credit['value'],
                        'currentBalance'  => $this->util->float($saldoAtual['VRSALDCONFAM'])
                    ];
                array_push($creditProcess,$finishcredit);
            }

            $fileName = date("Ymd-H-i-s") . "-" . $consumer['consumerRef'];
            $this->util->saveLogOrder(json_encode($creditProcess), 'BuyCredit/Processados/' . date("m/d"), $fileName);
            $this->util->commit();
            $response->addDataset(
                new DataSet(
                    '', 
                    $creditProcess
                )
            );

        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao realizar carga de credito. '.$e->getMessage();
            $fileName = date("Ymd-H-i-s") . "-" . $consumer['consumerRef'];
            $this->util->saveLogOrder($message, 'BuyCredit/Error/' . date("m/d"), $fileName);
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function registerConsumerPass(Request $request, Response $response){
        try{
            $body = $request->getParameters();
            $this->util->beginTransaction();

            if(!isset($body['consumerId'])) 
                throw new \Exception("Parâmetro consumerCode não informado!");
            
            if(!isset($body['password'])) 
                throw new \Exception("Parâmetro password não informado!");

            $consumer['consumerRef'] = $body['consumerId'];
            $password = $body['password'];
            $consumerDetails = $this->consumerService->getConsumerByRef($consumer);
            if(empty($consumerDetails) || $consumerDetails['error'])
                throw new \Exception($consumerDetails['message']);
            
            if(!empty($consumerDetails['data']['CDSENHACONS']))
                throw new \Exception("Consumidor {$consumerDetails['data']['NMCONSUMIDOR']} já possui senha cadastrada.");
            
            $passEncript = $this->consumerAPI->encryptDelphi($password);

            $passMD5 = md5($password);
            $result = $this->consumerService->saveConsumerPassword($consumerDetails['data'], $passEncript, $passMD5);
            if(empty($result) || $result['error'])
                throw new \Exception($result['message']);

            $this->util->commit();
            $response->addDataset(
                new DataSet(
                    '', 
                    $result
                )
            );

        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao cadastrar senha do consumidor. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function getConsumerByPhone(Request $request, Response $response){
        ini_set('memory_limit', '512M');

        try {
            $inputJSON = file_get_contents('php://input');
            $input = json_decode($inputJSON, true);

            if(isset($input['subsidiaryCode']))
                $subsidiaryCode = $input['subsidiaryCode'];
            else
                throw new \Exception('Código da filial não informado.'); 

            if (isset($input['consumerPhone']))
                $phone = preg_replace('/[^0-9]/', '', $input['consumerPhone']);
            else
                throw new \Exception('Telefone de consumidor não informado.');

            $consumer = $this->consumerService->getConsumerByPhone($phone);
            
            if (empty($consumer)) {
                $this->util->beginTransaction();
                try {
                    $defaultParams = $this->consumerService->getDefaultParams($subsidiaryCode);

                    if($defaultParams['error']) {
                        throw new \Exception($defaultParams['message']);
                    }

                    $consumer = [
                        'CDCLIENTE'       => $defaultParams['data']['CDCLIENTE'],
                        'CDCONSUMIDOR'    => str_pad($phone, 21, 0, STR_PAD_LEFT),
                        'NMCONSUMIDOR'    => 'NOVO CONSUMIDOR',
                        'CDCCUSCLIE'      => $defaultParams['data']['CDCCUSCLIE'],
                        'CDTIPOCONS'      => $defaultParams['data']['CDTIPOCONS'],
                        'CDIDCONSUMID'    => $phone,
                        'NRTELECONS'      => $phone,
                        'NRCPFRESPCON'    => null,
                        'DSEMAILCONS'     => null,
                        'DTNASCCONS'      => null,
                        'IDATIVO'         => '1',
                        'IDSEXOCONS'      => null,
                        'NRCEPCONS'       => null,
                        'DSENDECONS'      => null,
                        'NRENDECONS'      => null,
                        'CDMUNICIPIO'     => null,
                        'SGESTADO'        => null,
                        'CDPAIS'          => null,
                        'DSCOMPLENDECONS' => null,
                        'DSBAIRRO'        => null,
                        'NRCELULARCONS'   => $phone,
                        'IDATUCONSUMI'    => 'I'
                    ];

                    $registerConsumer = $this->consumerService->saveConsumer($consumer);
                    
                    if($registerConsumer['error']) {
                        throw new \Exception($registerConsumer['message']);
                    }
                    
                    $this->util->commit();
                } catch (\Exception $e) {
                    $this->util->rollBack();
                    throw new \Exception($e->getMessage());
                }
                
            }
            
            $return = [
                'clientCode'   => $consumer['CDCLIENTE'],
                'consumerCode' => $consumer['CDCONSUMIDOR'],
                'consumerName' => $consumer['NMCONSUMIDOR'],
                'consumerId'   => $consumer['CDIDCONSUMID']
            ];

            $response->addDataSet(
                new DataSet(
                    '',
                    $return
                )
            );
        } catch (\Exception $e) {
            $message = 'Erro na consulta de consumidor. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

}